unit MainForm;

{
Revision history:

V1.0.0  2000 Feb 19  New program
V1.0.2  2000 Feb 21  Add peak/RMS metering
                     Add input from file
V1.1.0  2001 Dec 31  Version for Delphi 5

Copyright  2000-2001 by David J Taylor, Edinburgh
Web site:  www.satsignal.net
E-mail:    davidtaylor@writeme.com
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WaveCommon, AudioAnalyser, Displays, StdCtrls, LED, MMsystem, ExtCtrls,
  Menus, ComCtrls, VersInfo;

type
  TForm1 = class (TForm)
    LevelMeter: TLevelMeter;
    AudioAnalyser: TBasicAudioAnalyser;
    ButtonRun: TButton;
    ButtonClose: TButton;
    LED1: TLED;
    Timer: TTimer;
    ShapeMeteor: TShape;
    ShapeNOAA: TShape;
    ShapeSICH: TShape;
    LabelMeteor: TLabel;
    LabelNOAA: TLabel;
    LabelSICH: TLabel;
    MainMenu1: TMainMenu;
    MenuHelp: TMenuItem;
    HelpAbout: TMenuItem;
    MenuFile: TMenuItem;
    FileExit: TMenuItem;
    LabelIntro: TLabel;
    CheckBoxPeak: TCheckBox;
    N1: TMenuItem;
    FileOpen: TMenuItem;
    OpenDialog: TOpenDialog;
    ProgressBar: TProgressBar;
    VI: TdfsVersionInfoResource;
    procedure AudioAnalyserClose (Sender: TObject);
    procedure AudioAnalyserData (var Header: wavehdr_tag);
    procedure AudioAnalyserOpen (Sender: TObject);
    procedure ButtonCloseClick (Sender: TObject);
    procedure ButtonRunClick (Sender: TObject);
    procedure CheckBoxPeakClick (Sender: TObject);
    procedure FileExitClick (Sender: TObject);
    procedure FileOpenClick (Sender: TObject);
    procedure FormCloseQuery (Sender: TObject; var CanClose: Boolean);
    procedure FormCreate (Sender: TObject);
    procedure FormPaint (Sender: TObject);
    procedure FormShow (Sender: TObject);
    procedure HelpAboutClick (Sender: TObject);
    procedure TimerTimer (Sender: TObject);
  private
    { Private declarations }
    left_offset: integer;
    right_offset: integer;
    total_samples: integer;
    from_file: boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses FormHelpAbout;

{$R *.DFM}
{$R \Davids\Delphi5\Lib\LED.dcr}

procedure TForm1.FormCreate (Sender: TObject);
begin
  AudioAnalyser.PeakLevel := CheckBoxPeak.Checked;
  if ParamCount > 0 then OpenDialog.FileName := ParamStr (1);
  AudioAnalyser.IgnoreFileSampleRate := True;
  Caption := 'SatLevel - V' + VI.FileVersion.AsString + ' - David J Taylor';
end;


procedure TForm1.FormCloseQuery (Sender: TObject; var CanClose: Boolean);
begin
  AudioAnalyser.Enabled := False;
end;


procedure TForm1.FormShow (Sender: TObject);

  function dB (value: real): integer;
  begin
    with LevelMeter do
      begin
      Result := Top + Round (Height * value / Abs (MindB));
      end;
  end;

var
  i: integer;
  dx, dy: integer;
begin
  ShapeMeteor.Top := dB (2.5);
  ShapeNOAA.Top := dB (3);
  ShapeSICH.Top := db (12);
  dy := Canvas.TextHeight ('Cy') div 2;
  dx := Canvas.TextWidth ('-00dB  ');
  with LevelMeter do
    for i := MaxdB div IntervaldB to -MindB div IntervaldB do
      begin
      with TLabel.Create (Self) do
        begin
        Parent := Form1;
        Top := LevelMeter.Top + i * LevelMeter.Height * IntervaldB div -MindB - dy;
        AutoSize := True;
        Alignment := taRightJustify;
        Caption := IntToStr (-IntervaldB * i) + 'dB  ';
        Left := LevelMeter.Left - dx;
        end;
      end;
  Timer.Enabled := True;
end;


procedure TForm1.ButtonCloseClick (Sender: TObject);
begin
  Close;
end;


procedure TForm1.ButtonRunClick (Sender: TObject);
begin
  if AudioAnalyser.Enabled
  then
    begin
    AudioAnalyser.Enabled := False;
    ButtonRun.Caption := '&Start';
    end
  else
    begin
    AudioAnalyser.Enabled := True;
    ButtonRun.Caption := '&Stop';
    end;
end;


procedure TForm1.AudioAnalyserOpen (Sender: TObject);
begin
  LED1.Colour := clLime;
end;


procedure TForm1.AudioAnalyserClose (Sender: TObject);
begin
  LED1.Colour := clGreen;
end;


procedure TForm1.AudioAnalyserData (var Header: wavehdr_tag);
// This routine computes an average offset and feeds it
// back to make the long-term mean offset zero.
var
  samples: integer;
  signal: PAudioSample;
  left, right: integer;
  sample: integer;
  smooth: integer;
begin
  // Extract the number of samples from the wave header
  samples := Header.dwBytesRecorded div SizeOf (TStereoPair);
  Inc (total_samples, samples);
  // and point to the buffer in its actual format
  signal := PAudioSample (Header.lpData);
  left := 0;
  right := 0;
  for sample := 0 to samples - 1 do
    begin
    left := left + signal^;
    Inc (signal);
    right := right + signal^;
    Inc (signal);
    end;
  left := left div samples;
  right := right div samples;
  if total_samples < 44100
  then smooth := 1
  else if total_samples < 441000
       then smooth := 4
       else smooth := 16;
  left_offset := left_offset + left div smooth;
  right_offset := right_offset + right div smooth;
  AudioAnalyser.LeftOffset := left_offset;
  AudioAnalyser.RightOffset := right_offset;
  if from_file then
    begin
    ProgressBar.Position :=
      100 * AudioAnalyser.BufferNumber div AudioAnalyser.BufferCount;
    ProgressBar.Update;
    Application.ProcessMessages;
    Sleep (20);
    end;
end;


procedure TForm1.TimerTimer (Sender: TObject);
begin
  Timer.Enabled := False;
  ButtonRun.Click;
end;


procedure TForm1.FormPaint (Sender: TObject);
begin
  with Canvas do
    begin
    MoveTo (ShapeMeteor.Left + ShapeMeteor.Width, ShapeMeteor.Top);
    LineTo (LabelMeteor.Left, LabelMeteor.Top + (TextHeight ('Cy') div 2));
    MoveTo (ShapeNOAA.Left + ShapeNOAA.Width, ShapeNOAA.Top);
    LineTo (LabelNOAA.Left, LabelNOAA.Top + (TextHeight ('Cy') div 2));
    MoveTo (ShapeSICH.Left + ShapeSICH.Width, ShapeSICH.Top);
    LineTo (LabelSICH.Left, LabelSICH.Top + (TextHeight ('Cy') div 2));
    end;
end;

procedure TForm1.FileExitClick (Sender: TObject);
begin
  Close;
end;


procedure TForm1.HelpAboutClick (Sender: TObject);
begin
  HelpAboutBox.ShowModal;
end;


procedure TForm1.CheckBoxPeakClick (Sender: TObject);
begin
  AudioAnalyser.PeakLevel := CheckBoxPeak.Checked;
end;


procedure TForm1.FileOpenClick (Sender: TObject);
begin
  if AudioAnalyser.Enabled then ButtonRun.Click;
  with OpenDialog do
    begin
    ButtonRun.Enabled := False;
    if Execute then
      begin
      ProgressBar.Visible := True;
      from_file := True;
      AudioAnalyser.LoadFromFile (Filename);
      from_file := False;
      ProgressBar.Visible := False;
      end;
    ButtonRun.Enabled := True;
    end;
end;


end.

